@@ -46,11 +46,12 @@
 
 :- autoload(library(ansi_term),[ansi_format/3]).
 :- autoload(library(apply),[foldl/4]).
-:- autoload(library(debug),[debug/3]).
+:- use_module(library(debug),[debug/3]).
 :- autoload(library(error),[instantiation_error/1,must_be/2]).
 :- autoload(library(lists),[member/2]).
 :- autoload(library(option),[option/2,option/3,meta_options/3]).
 :- autoload(library(prolog_clause),[clause_info/5]).
+:- autoload(library(prolog_code), [most_general_goal/2]).
 
 %:- set_prolog_flag(generate_debug_info, false).
 
@@ -162,6 +163,8 @@ list_module(Module, Options) :-
 %       lists:append([], L, L).
 %       ==
 %
+%     * A clause reference as obtained for example from nth_clause/3.
+%
 %    The following options are defined:
 %
 %      - variable_names(+How)
@@ -203,6 +206,10 @@ listing_(M:List, Options) :-
     !,
     forall(member(Spec, List),
            listing_(M:Spec, Options)).
+listing_(M:CRef, Options) :-
+    blob(CRef, clause),
+    !,
+    list_clauserefs([CRef], M, Options).
 listing_(X, Options) :-
     (   prolog:locate_clauses(X, ClauseRefs)
     ->  strip_module(X, Context, _),
@@ -217,8 +224,8 @@ list_clauserefs([H|T], Context, Options) :-
     list_clauserefs(H, Context, Options),
     list_clauserefs(T, Context, Options).
 list_clauserefs(Ref, Context, Options) :-
-    @(rule(_, Rule, Ref), Context),
-    list_clause(Rule, Ref, Context, Options).
+    @(rule(M:_, Rule, Ref), Context),
+    list_clause(M:Rule, Ref, Context, Options).
 
 %!  list_predicates(:Preds:list(pi), :Spec, +Options) is det.
 
@@ -388,6 +395,8 @@ rule_head((Head0 => _Body), Head) :- !, Head = Head0.
 rule_head(?=>(Head0, _Body), Head) :- !, Head = Head0.
 rule_head(Head, Head).
 
+%!  list_clause(+Term, +ClauseRef, +ContextModule, +Options)
+
 list_clause(_Rule, Ref, _Source, Options) :-
     option(source(true), Options),
     (   clause_property(Ref, file(File)),
@@ -730,11 +739,12 @@ do_portray_clause(Out, (:-Directive), Options) :-
     nlindent(Out, Indent),
     portray_list(List, Indent, Out, Options),
     write(Out, ').\n').
-do_portray_clause(Out, (:-Directive), Options) :-
+do_portray_clause(Out, Clause, Options) :-
+    directive(Clause, Op, Directive),
     !,
     option(indent(LeftMargin), Options, 0),
     indent(Out, LeftMargin),
-    write(Out, ':- '),
+    format(Out, '~w ', [Op]),
     DIndent is LeftMargin+3,
     portray_body(Directive, DIndent, noindent, 1199, Out, Options),
     full_stop(Out).
@@ -753,6 +763,9 @@ full_stop(Out) :-
     '$put_token'(Out, '.'),
     nl(Out).
 
+directive((:- Directive), :-, Directive).
+directive((?- Directive), ?-, Directive).
+
 wrapped_list_directive(module(_,_)).
 %wrapped_list_directive(use_module(_,_)).
 %wrapped_list_directive(autoload(_,_)).
@@ -1069,7 +1082,7 @@ pprint(Out, Term, Pri, Options) :-
     ;   is_dict(Term)
     ),
     \+ nowrap_term(Term),
-    setting(listing:line_width, Width),
+    line_width(Width),
     Width > 0,
     (   write_length(Term, Len, [max_length(Width)|Options])
     ->  true
@@ -1088,6 +1101,7 @@ pprint(Out, Term, Pri, Options) :-
                | WrtOptions
                ]).
 
+:- public portray_blob/2.
 portray_blob(Blob, _Options) :-
     blob(Blob, _),
     \+ atom(Blob),
@@ -1127,7 +1141,7 @@ pprint_wrapped(Out, Dict, _, Options) :-
     format(Out, '}', []).
 pprint_wrapped(Out, Term, _, Options) :-
     Term =.. [Name|Args],
-    format(Out, '~q(', Name),
+    format(Out, '~q(', [Name]),
     line_position(Out, Indent),
     pprint_args(Args, Indent, Out, Options),
     format(Out, ')', []).
@@ -1198,6 +1212,16 @@ put_tabs(Out, N) :-
     put_tabs(Out, NN).
 put_tabs(_, _).
 
+line_width(Width) :-
+    stream_property(current_output, tty(true)),
+    catch(tty_size(_Rows, Cols), error(_,_), fail),
+    !,
+    Width is Cols - 2.
+line_width(Width) :-
+    setting(listing:line_width, Width),
+    !.
+line_width(78).
+
 
 %!  inc_indent(+Indent0, +Inc, -Indent)
 %
