@@ -3,7 +3,7 @@
     Author:        Jeffrey Rosenwald, extended by Peter Ludemann
     E-mail:        jeffrose@acm.org, peter.ludemann@gmail.com
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2010-2015, Jeffrey Rosenwald
+    Copyright (c)  2010-2015, Jeffrey Rosenwald; 2021-2024, SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -434,8 +434,8 @@ static foreign_t
 int32_float32(term_t Int32, term_t Float32)
 {
   /* TODO: might generate better code if we do
-              asInt = (int32_t)asFloat
-              asFloat = (double)asInt
+              int32_t asInt = (int32_t)asFloat
+              float asFloat = (float)asInt
    */
   union
   {
