@@ -2,7 +2,7 @@
 
     Author:        Matt Lilley and Markus Triska
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2004-2024, SWI-Prolog Foundation
+    Copyright (c)  2004-2025, SWI-Prolog Foundation
                               VU University Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -36,6 +36,7 @@
 #define _CRT_SECURE_NO_WARNINGS 1
 #include <config.h>
 #include <assert.h>
+#include <stdbool.h>
 #include <string.h>
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
@@ -52,6 +53,7 @@
 #endif
 #include "crypt_blowfish.h"
 
+#include "common.h"
 #include "cryptolib.c"
 
 static atom_t ATOM_sslv23;
@@ -240,7 +242,7 @@ unify_hash_context(term_t tcontext, PL_CRYPTO_HASH_CONTEXT *context)
 }
 
 
-static int
+static bool
 get_hash_context(term_t tcontext, PL_CRYPTO_HASH_CONTEXT **context)
 { PL_blob_t *type;
   void *data;
@@ -252,10 +254,10 @@ get_hash_context(term_t tcontext, PL_CRYPTO_HASH_CONTEXT **context)
     assert(c->magic == HASH_CONTEXT_MAGIC);
     *context = c;
 
-    return TRUE;
+    return true;
   }
 
-  return PL_type_error("crypto_hash_context", tcontext);
+  return PL_type_error("crypto_hash_context", tcontext),false;
 }
 
 typedef struct algorithm_pair {
@@ -291,7 +293,7 @@ get_hash_algorithm(atom_t a_algorithm, const EVP_MD **algorithm)
   return FALSE;
 }
 
-static int
+static bool
 get_text_representation(term_t t, int *rep)
 { atom_t a;
 
@@ -299,12 +301,12 @@ get_text_representation(term_t t, int *rep)
   { if      ( a == ATOM_octet ) *rep = REP_ISO_LATIN_1;
     else if ( a == ATOM_utf8  ) *rep = REP_UTF8;
     else if ( a == ATOM_text  ) *rep = REP_MB;
-    else return PL_domain_error("encoding", t);
+    else return PL_domain_error("encoding", t),false;
 
-    return TRUE;
+    return true;
   }
 
-  return FALSE;
+  return false;
 }
 
 
@@ -892,7 +894,7 @@ recover_rsa(term_t t, RSAKEY** keyp)
   RSAKEY *key = RSA_new();
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if SSL_API_0
   if ( get_bn_arg(1, t, &key->n) &&
        get_bn_arg(2, t, &key->e) &&
        get_bn_arg(3, t, &key->d) &&
@@ -1118,8 +1120,8 @@ pl_ecdsa_sign(term_t Private, term_t Data, term_t Enc, term_t Signature)
   EVP_PKEY_CTX *sign_ctx = EVP_PKEY_CTX_new(key, NULL);
   EVP_PKEY_sign_init(sign_ctx);
   rc = EVP_PKEY_sign(sign_ctx,
-		                 signature, &signature_len,
-		                 data, (unsigned int)data_len);
+				 signature, &signature_len,
+				 data, (unsigned int)data_len);
   EVP_PKEY_CTX_free(sign_ctx);
   if (!rc)
     return raise_ssl_error(ERR_get_error());
