@@ -64,7 +64,7 @@
     epilog(:),
     set_epilog(:),
     run_in_help_epilog(0),
-    epilog_insert_menu_item(+, +, +, 0).
+    win_insert_menu_item(+, +, +, 0).
 
 :- pce_autoload(finder, library(find_file)).
 :- pce_global(@finder, new(finder)).
@@ -292,6 +292,10 @@ binding('\\C-\\S-w', close).
 binding('\\C-\\S-=', font_magnify).
 binding('\\C--',     font_reduce).
 binding('\\C-=',     font_default).
+binding('<f5>',      trace_mode).
+binding('\\S-<f5>',  debug_mode).
+binding('\\C-<f5>',  gui_debug).
+binding('<f6>',      debugging).
 :- if(current_prolog_flag(apple, true)).
 binding(Key, Method) :-
     pce_keybinding:binding(apple, epilog, Bindings),
@@ -494,6 +498,45 @@ interrupt(PT) :->
     current_signal(int, SIGINT, debug),
     thread_signal(Thread, SIGINT).
 
+debug_mode(PT) :->
+    "Toggle Prolog debug mode"::
+    (   terminal_prolog_flag(PT, query_debug_settings,
+                             debug(Debugging, _Tracing), -)
+    ->  debug_toggle_command(Debugging, Negate),
+        send(PT, inject, Negate)
+    ;   true
+    ).
+
+debug_toggle_command(true, nodebug).
+debug_toggle_command(false, debug).
+
+trace_mode(PT) :->
+    "Toggle Prolog trace mode"::
+    (   terminal_prolog_flag(PT, query_debug_settings,
+                             debug(_Debugging, Tracing), -)
+    ->  trace_toggle_command(Tracing, Negate),
+        send(PT, inject, Negate)
+    ;   true
+    ).
+
+trace_toggle_command(true, notrace).
+trace_toggle_command(false, trace).
+
+debugging(PT) :->
+    "Show debugging status"::
+    send(PT, inject, debugging).
+
+gui_debug(PT) :->
+    "Toggle Prolog GUI tracer"::
+    (   terminal_prolog_flag(PT, gui_tracer, GuiDebug, false)
+    ->  gui_debug_toggle_command(GuiDebug, Negate),
+        send(PT, inject, Negate)
+    ;   true
+    ).
+
+gui_debug_toggle_command(true,  noguitracer).
+gui_debug_toggle_command(false, guitracer).
+
 close(PT) :->
     "Close this Prolog shell"::
     get(PT, window, Window),
@@ -1015,6 +1058,75 @@ open_url(_T, URL:name) :->
     "Open a URL"::
     www_open_url(URL).
 
+debug_mode(Frame) :->
+    "Toggle Prolog debug mode"::
+    get(Frame, current_terminal, Term),
+    send(Term, debug_mode).
+
+trace_mode(Frame) :->
+    "Toggle Prolog trace mode"::
+    get(Frame, current_terminal, Term),
+    send(Term, trace_mode).
+
+debugging(Frame, MI:menu_item) :->
+    "Toggle Prolog debug mode"::
+    get(Frame, current_terminal, Term),
+    send(Term, debugging),
+    send(MI, selected, @off).
+
+gui_debug(Frame) :->
+    "Toggle GUI tracer"::
+    get(Frame, current_terminal, Term),
+    send(Term, gui_debug).
+
+update_debug_mode(Frame, MI:menu_item) :->
+    "Updated the current debug mode"::
+    (   epilog_prolog_flag(Frame, query_debug_settings,
+                           debug(Debugging, _Tracing), -)
+    ->  send(MI, selected, Debugging)
+    ;   true
+    ).
+
+update_trace_mode(Frame, MI:menu_item) :->
+    "Updated the current trace mode"::
+    (   epilog_prolog_flag(Frame, query_debug_settings,
+                           debug(_Debugging, Tracing), -)
+    ->  send(MI, selected, Tracing)
+    ;   true
+    ).
+
+update_gui_debug(Frame, MI:menu_item) :->
+    (   epilog_prolog_flag(Frame, gui_tracer, GuiTracer, false)
+    ->  send(MI, selected, GuiTracer)
+    ;   true
+    ).
+
+epilog_prolog_flag(Frame, Flag, Value, Default) :-
+    get(Frame, current_terminal, Term),
+    terminal_prolog_flag(Term, Flag, Value, Default).
+
+%!  terminal_prolog_flag(+Term, +Flag, -Value, +Default) is semidet.
+%
+%   Get the Prolog flat Flag for the toplevel thread running in Term. If
+%   the flag is not defined, unify   Value  with Default. This predicate
+%   uses a timeout of 0.1 seconds,   returning  Default on timeout. This
+%   guarantees that the console will not  freeze   if  the thread is not
+%   responsive.
+
+terminal_prolog_flag(Term, Flag, Value, Default) :-
+    current_prolog_terminal(Thread, Term),
+    (   catch(call_in_thread(Thread,
+                             current_prolog_flag(Flag, Value),
+                             [ timeout(0.1),
+                               on_timeout(fail)
+                             ]),
+              error(Formal,_),
+              true)
+    ->  var(Formal)
+    ;   Value = Default
+    ).
+
+
 :- pce_end_class(epilog_frame).
 
 :- pce_begin_class(epilog_dialog, dialog, "Prolog terminator menu").
@@ -1027,6 +1139,7 @@ initialise(D) :->
     send(MB, append, new(File,     popup(file))),
     send(MB, append, new(Settings, popup(settings))),
     send(MB, append, new(Tools,    popup(tools))),
+    send(MB, append, new(Debug,    epilog_popup(debug))),
     send(MB, append, new(Help,     popup(help))),
     Epilog = @event?receiver?frame,
     send_list(File, append,
@@ -1061,6 +1174,24 @@ initialise(D) :->
                 menu_item(cross_referencer,
                           message(Epilog, ide, xref))
               ]),
+    send_list(Debug, append,
+              [ new(TraceMode,
+                    menu_item(trace_mode,
+                              message(Epilog, trace_mode),
+                              accelerator := 'F5')),
+                new(DebugMode,
+                    menu_item(debug_mode,
+                              message(Epilog, debug_mode),
+                              accelerator := 'Shift-F5')),
+                new(GuiDebug,
+                    menu_item('GUI_debugger',
+                              message(Epilog, gui_debug),
+                              accelerator := 'Ctrl-F5',
+                              end_group := @on)),
+                menu_item(show_debug_status,
+                          message(Epilog, debugging, @arg1),
+                          accelerator := 'F6')
+              ]),
     send_list(Help, append,
               [ menu_item('SWI-Prolog documentation',
                           message(Epilog, open_url,
@@ -1069,10 +1200,15 @@ initialise(D) :->
                           message(Epilog, open_url,
                                   'https://swi-prolog.discourse.group/'),
                           end_group := @on),
-                menu_item('XPCE on SDL+Cairo',
+                menu_item('SWI-Prolog GUI tools',
                           message(Epilog, open_url,
                                   'https://github.com/SWI-Prolog/packages-xpce/wiki'))
-              ]).
+              ]),
+    send(Debug, show_current, @on),
+    send(Debug, multiple_selection, @on),
+    send(DebugMode, condition, message(Epilog, update_debug_mode, DebugMode)),
+    send(TraceMode, condition, message(Epilog, update_trace_mode, TraceMode)),
+    send(GuiDebug,  condition, message(Epilog, update_gui_debug, GuiDebug)).
 
 :- pce_end_class(epilog_dialog).
 
@@ -1111,13 +1247,10 @@ capture_messages :-
 
 :- public xpce_message/3.
 xpce_message(_Term, Kind, Lines) :-
-    report_kind(Kind),
+    Kind \== silent,
     xpce_epilog_console(_In,_Out,Error),
     print_message_lines(Error, kind(Kind), Lines).
 
-report_kind(warning).
-report_kind(error).
-
 %!  pce:xpce_console(-In,-Out,-Error) is semidet.
 %
 %   Tell xpce where to write console output.   If  this fails, output is
