@@ -594,7 +594,7 @@ toInt(intptr_t i)
 #define isFreedObj(obj)		onFlag(obj, F_FREED)
 #define setFreeingObj(obj)	setFlag(obj, F_FREEING)
 #define isFreeingObj(obj)	onFlag(obj, F_FREEING)
-#define isVirginObj(o)		(noRefsObj(o) && \
+#define isVirginObj(o)		(refsObject(o) == 0 && \
 				 !onFlag(o, F_LOCKED|F_PROTECTED|F_ANSWER))
 #define freeableObj(o)		if ( isVirginObj(o) ) \
 				  freeObject(o)
@@ -1546,7 +1546,6 @@ struct to_cell
   long		index;			/* Index of the mark */
 };
 
-GLOBAL ToCell	AnswerStack;		/* Stack of `answer objects' */
 GLOBAL int	deferredUnalloced;	/* # deferred unallocs in ->free */
 
 typedef intptr_t AnswerMark;
@@ -1771,9 +1770,14 @@ extern char *T_report[];		/* ->report: kind, format, args... */
   }
 
 
-#define for_cell(c, ch)	for(c=(ch)->head; notNil(c); c=c->next)
-#define for_cell_save(p, q, ch)	if (notNil(p=(ch)->head))\
-		for(q=p->next; notNil(p); p=q, q=(isNil(q) ? q : q->next))
+/* Iterate over a chain.  The body may remove the cell `c`, but __not__
+ * the cell that follows `c`.
+ */
+
+#define for_cell(c, ch)							\
+  if (notNil(c=(ch)->head))						\
+    for(Cell _q=c->next; notNil(c); c=_q, _q=(isNil(_q) ? _q : _q->next))
+
 
 		/********************************
 		*          EXPRESSIONS		*
