@@ -372,11 +372,12 @@ callExitMessagesPce(int stat, Pce pce)
 { static int done = 0;
 
   if ( !done++ && pce && notNil(pce) )
-  { Cell cell, q;
+  { Cell cell;
 
-    for_cell_save(cell, q, pce->exit_messages)
+    for_cell(cell, pce->exit_messages)
     { addCodeReference(cell->value);
       forwardCode(cell->value, toInt(stat), EAV);
+      delCodeReference(cell->value);
     }
   }
 }
@@ -1535,7 +1536,6 @@ pceInitialise(int handles, const char *home, const char *appdata,
   PCEargv = argv;
 
   MaxGoalDepth = NO_MAX_GOAL_DEPTH;
-  initAnswerStack();
   initMClock();
 
 #ifndef O_RUNTIME
