@@ -166,13 +166,16 @@ ws_create_frame(FrameObj fr)
   int y = valInt(fr->area->y);
   int w = valInt(fr->area->w);
   int h = valInt(fr->area->h);
+  bool focusable = true;
 
   if ( fr->kind == NAME_popup && parent )
-  { SDL_PropertiesID props = SDL_CreateProperties();
+  { focusable = false;
+    SDL_PropertiesID props = SDL_CreateProperties();
     SDL_SetPointerProperty(props, SDL_PROP_WINDOW_CREATE_PARENT_POINTER,
 			   parent);
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_MENU_BOOLEAN, true);
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_HIDDEN_BOOLEAN, true);
+    SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_FOCUSABLE_BOOLEAN, false);
 #if defined(__APPLE__) && defined(SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN)
     /* SDL on MacOS does not handle popup placement correctly on secondary displays */
     SDL_SetBooleanProperty(props, SDL_PROP_WINDOW_CREATE_CONSTRAIN_POPUP_BOOLEAN, false);
@@ -267,7 +270,8 @@ ws_create_frame(FrameObj fr)
 
     ws_draw_frame(fr);
     SDL_ShowWindow(win);
-    SDL_RaiseWindow(win);
+    if ( focusable )
+      SDL_RaiseWindow(win);
 
     succeed;
   }
@@ -644,22 +648,34 @@ sdl_frame_event(SDL_Event *ev)
       }
       case SDL_EVENT_WINDOW_FOCUS_GAINED:
       { PceWindow sw = ws_grabbing_window();
-	if ( sw && getFrameWindow(sw, OFF) != fr )
-	{ WsFrame wfr = fr->ws_ref;
-	  SDL_StartTextInput(wfr->ws_window);
-	  return true;
+	if ( sw )
+	{ FrameObj fr2 = getFrameWindow(sw, OFF);
+
+	  DEBUG(NAME_keyboard,
+		Cprintf("Input focus on %s (grabbing=%s on %s)\n",
+			pp(fr), pp(sw), pp(fr2)));
+
+	  if ( fr2 != fr )
+	  { WsFrame wfr = fr->ws_ref;
+	    SDL_StartTextInput(wfr->ws_window);
+	    return true;
+	  }
+	} else
+	{ DEBUG(NAME_keyboard,
+		Cprintf("Input focus on %s (not grabbing)\n",
+			pp(fr)));
 	}
-	DEBUG(NAME_keyboard, Cprintf("Input focus on %s\n", pp(fr)));
 	return send(fr, NAME_inputFocus, ON, EAV);
       }
       case SDL_EVENT_WINDOW_FOCUS_LOST:
       { PceWindow sw = ws_grabbing_window();
+	DEBUG(NAME_keyboard, Cprintf("Input focus lost for %s (grabbing=%s)\n",
+				     pp(fr), pp(sw)));
 	if ( sw && getFrameWindow(sw, OFF) != fr )
 	{ WsFrame wfr = fr->ws_ref;
 	  SDL_StopTextInput(wfr->ws_window);
 	  return true;
 	}
-	DEBUG(NAME_keyboard, Cprintf("Input focus lost for %s\n", pp(fr)));
 	return send(fr, NAME_inputFocus, OFF, EAV);
       }
       case SDL_EVENT_WINDOW_DISPLAY_CHANGED:
