@@ -2541,7 +2541,6 @@ rlc_erase_display(RlcData b)
 
   RlcTextLine tl = &b->lines[b->window_start];
   tl->size = 0;
-  tl->adjusted = false;
   b->last = b->window_start;
   b->changed |= CHG_CHANGED|CHG_CLEAR|CHG_CARET;
 
@@ -3281,7 +3280,7 @@ rlc_putansi(RlcData b, int chr)
 	    break;
 	  }
 	case '@':		/* \e[<n>@: insert <n> spaces */
-	  rlc_need_arg(b, 1, 0);
+	  rlc_need_arg(b, 1, 1);
 	  for(int i=0; i<b->argv[0]; i++)
 	    rlc_insert(b, ' ');
 	  break;
@@ -3442,13 +3441,13 @@ processClientOutputTerminalImage(TerminalImage ti,
       buf -= b->incomplete_cnt;
       count += b->incomplete_cnt;
       memcpy(buf, b->incomplete, b->incomplete_cnt);
+      b->incomplete_cnt = 0;
     }
 
     const char *i = buf;
     const char *end = &buf[count];
-    bool debug = false;
     if ( debug ) Cprintf("Received (%d bytes): ", count);
-    for(;;)
+    while( i < end )
     { char c = *i;
       int sz = UTF8_FBN(c);
       if ( sz < 0 )		 /* Invalid UTF-8 */
@@ -3672,7 +3671,7 @@ rlc_create_pipes(RlcData b)
   if ( !CreatePipeEx(&b->ptycon.hIn, &b->ptycon.hTaskOut, NULL,
 		     PIPE_SIZE, FILE_FLAG_OVERLAPPED, 0) ||
        !CreatePipeEx(&b->ptycon.hTaskIn, &b->ptycon.hOut,  NULL,
-		     PIPE_SIZE, 0, FILE_FLAG_OVERLAPPED) )
+		     PIPE_SIZE, FILE_FLAG_OVERLAPPED, FILE_FLAG_OVERLAPPED) )
   { Cprintf("Failed to create ptyCon pipes\n");
     return false;
   }
