@@ -232,8 +232,8 @@ getConfirmFrame(FrameObj fr, Point pos, BoolObj grab)
   if ( isObject(rval) )
   { addCodeReference(rval);
     assign(fr, return_value, ConstantNotReturned);
-    delCodeReference(rval);
     pushAnswerObject(rval);
+    delCodeReference(rval);
   } else
     assign(fr, return_value, ConstantNotReturned);
 
@@ -535,8 +535,9 @@ frame_is_upto_date(FrameObj fr)
 
 
 static status
-waitFrame(FrameObj fr)
-{ if ( fr->status == NAME_unmapped )
+SdlWaitFrame(FrameObj fr)
+{ ASSERT_SDL_MAIN();
+  if ( fr->status == NAME_unmapped )
     TRY(send(fr, NAME_open, EAV));
 
   while( !frame_is_upto_date(fr) )
@@ -551,6 +552,17 @@ waitFrame(FrameObj fr)
 }
 
 
+static status
+waitFrame(FrameObj fr)
+{ if ( isOpenFrameStatus(fr->status) )
+    succeed;
+
+  if ( SDL_IsMainThread() )
+    return SdlWaitFrame(fr);
+  else
+    return wait_host(true, fr, NAME_SdlWait, EAV);
+}
+
 static status
 showFrame(FrameObj fr, BoolObj val)
 { if ( val == ON )
@@ -1828,6 +1840,8 @@ static senddecl send_frame[] =
      NAME_open, "Destroy window-system counterpart"),
   SM(NAME_wait, 0, NULL, waitFrame,
      NAME_open, "Wait till <-status is `open'"),
+  SM(NAME_SdlWait, 0, NULL, SdlWaitFrame,
+     NAME_open, "SDL main thread helper for frame->wait"),
   SM(NAME_append, 1, "subwindow=window", appendFrame,
      NAME_organisation, "Append a window to the frame"),
   SM(NAME_delete, 1, "member:window", deleteFrame,
