@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2004-2020, University of Amsterdam
+    Copyright (c)  2004-2024, University of Amsterdam
                               VU University Amsterdam
+			      SWI-Prolog Solutions b.v.
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -47,8 +48,10 @@
 
 void	destroyGlobalVars(void);
 void	freezeGlobal(void);
-int	gvar_value(atom_t name, Word p);
+bool	gvar_value(atom_t name, Word p);
 
 #undef LDFUNC_DECLARATIONS
 
-#endif /*_PL_GVAR_H*/
\ No newline at end of file
+int	PL_getval(atom_t name, term_t val, unsigned int flags);
+
+#endif /*_PL_GVAR_H*/
