@@ -699,9 +699,7 @@ setupGNUEmacsInferiorMode(void)
   if ( ((s = Getenv("EMACS", envbuf, sizeof(envbuf))) && s[0]) ||
        ((s = Getenv("INSIDE_EMACS", envbuf, sizeof(envbuf))) && s[0]) ||
        ((s = Getenv("INFERIOR", envbuf, sizeof(envbuf))) && streq(s, "yes")) )
-  { GET_LD
-
-    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+  { PL_set_prolog_flag("tty_control", PL_BOOL, false);
     val = true;
 #ifdef __WINDOWS__
     Sinput->flags  |= SIO_ISATTY;
@@ -841,10 +839,12 @@ initDefaults(void)
   GD->signals.sig_alert      = SIG_ALERT;
 #endif
 
-  if ( systemDefaults.notty )
-    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
-  else
-    setPrologFlagMask(PLFLAG_TTY_CONTROL);
+  if ( !current_prolog_flag("tty_control") )
+  { if ( systemDefaults.notty )
+      clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+    else
+      setPrologFlagMask(PLFLAG_TTY_CONTROL);
+  }
 
   setPrologFlagMask(PLFLAG_DEBUGINFO);
   setPrologFlagMask(PLFLAG_GCTHREAD);
@@ -1018,11 +1018,8 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 	  return -1;
       } else if ( (rc=is_bool_opt(s, "tty", &b)) )
       { if ( rc == true )
-	{ if ( b )
-	    setPrologFlagMask(PLFLAG_TTY_CONTROL);
-	  else
-	    clearPrologFlagMask(PLFLAG_TTY_CONTROL);
-	} else
+	  PL_set_prolog_flag("tty_control", PL_BOOL, b);
+	else
 	  return -1;
       } else if ( (rc=is_bool_opt(s, "pce", &b)) )
       { if ( rc == true )
@@ -1098,7 +1095,7 @@ parseCommandLineOptions(int argc0, char **argv0, char **argvleft)
 
     if ( streq(s, "tty") )
     { Sdprintf("Warning: `-tty` is deprecated.  Please use `--no-tty`\n");
-      clearPrologFlagMask(PLFLAG_TTY_CONTROL);
+      PL_set_prolog_flag("tty_control", PL_BOOL, false);
       continue;
     }
 
@@ -1436,6 +1433,13 @@ PL_initialise(int argc, char **argv)
   GD->cmdline.appl_argc = argc;
   GD->cmdline.appl_argv = argv;
 
+#ifdef __WINDOWS__
+  if ( truePrologFlag(PLFLAG_TTY_CONTROL) &&
+       !truePrologFlag(PLFLAG_EPILOG) )
+    PL_w32_wrap_ansi_console();	/* decode ANSI color sequences (ESC[...m) */
+  else
+    PL_set_prolog_flag("color_term", PL_BOOL, false);
+#endif
   setupGNUEmacsInferiorMode();		/* Detect running under EMACS */
 
   if ( !setupProlog() )
