@@ -756,7 +756,7 @@ typedef struct attvar_mark
   Word   attvar;
 } attvar_mark;
 
-int
+int	/* bool|TRIE_ABSTRACTED|TRIE_LOOKUP_CONTAINS_ATTVAR|TRIE_LOOKUP_CYCLIC */
 trie_lookup_abstract(DECL_LD trie *trie, trie_node *node, trie_node **nodep,
 		     Word k, bool add, size_abstract *abstract,
 		     TmpBuffer vars)
@@ -1495,7 +1495,7 @@ trie_delete(trie *trie, trie_node *node, int prune)
  */
 
 #define trie_insert(Trie, Key, Value, nodep, update, abstract) LDFUNC(trie_insert, Trie, Key, Value, nodep, update, abstract)
-static int
+static bool
 trie_insert(DECL_LD term_t Trie, term_t Key, term_t Value, trie_node **nodep,
 	    int update, size_abstract *abstract)
 { trie *trie;
@@ -1513,7 +1513,7 @@ trie_insert(DECL_LD term_t Trie, term_t Key, term_t Value, trie_node **nodep,
     } else
     { if ( (Value  && isoff(trie, TRIE_ISMAP)) ||
 	   (!Value && isoff(trie, TRIE_ISSET)) )
-      { return PL_permission_error("insert", "trie", Trie);
+      { return PL_permission_error("insert", "trie", Trie),false;
       }
     }
 
@@ -1646,7 +1646,7 @@ PRED_IMPL("trie_update", 3, trie_update, 0)
 static
 PRED_IMPL("trie_insert", 4, trie_insert, 0)
 { PRED_LD
-  trie_node *node;
+  trie_node *node = NULL;	/* avoid warning */
 
   return ( trie_insert(A1, A2, A3, &node, false, NULL) &&
 	   PL_unify_pointer(A4, node) );
@@ -1966,7 +1966,12 @@ unify_key(DECL_LD ukey_state *state, word key)
 
 	  deRef2(vi->address, p2);
 	  if ( !isVar(*p2) )
-	  { assignAttVar(vi->attributes, p2);
+	  { if ( ison(state->trie, TRIE_ISTABLE) )
+	    { TrailAssignment(p2);
+	      *p2 = makeRefG(vi->attributes);
+	    } else
+	    { assignAttVar(vi->attributes, p2);
+	    }
 	  } else
 	  { int rc = unify_ptrs(vi->attributes, vi->address, ALLOW_RETCODE);
 	    if ( rc != true )
@@ -2438,7 +2443,7 @@ unify_trie_path(DECL_LD term_t term, trie_node **tn, trie_gen_state *gstate)
 
 foreign_t
 trie_gen_raw(trie *trie, trie_node *root, term_t Key, term_t Value,
-	     term_t Data, int LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
+	     term_t Data, bool LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
 	     void *ctx, control_t PL__ctx)
 { PRED_LD
   trie_gen_state state_buf;
@@ -2556,7 +2561,7 @@ next:;
 
 foreign_t
 trie_gen(term_t Trie, term_t Root, term_t Key, term_t Value,
-	 term_t Data, int LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
+	 term_t Data, bool LDFUNCP (*unify_data)(DECL_LD term_t, trie_node*, void *ctx),
 	 void *ctx, control_t PL__ctx)
 { if ( CTX_CNTRL == FRG_FIRST_CALL )
   { trie *trie;
@@ -2595,8 +2600,10 @@ PRED_IMPL("trie_gen", 2, trie_gen, PL_FA_NONDETERMINISTIC)
 { return trie_gen(A1, 0, A2, 0, 0, NULL, NULL, PL__ctx);
 }
 
-#define unify_node_id(t, answer, ctx) LDFUNC(unify_node_id, t, answer, ctx)
-static int
+#define unify_node_id(t, answer, ctx) \
+	LDFUNC(unify_node_id, t, answer, ctx)
+
+static bool
 unify_node_id(DECL_LD term_t t, trie_node *answer, void *ctx)
 { (void) ctx;
 
@@ -3172,7 +3179,12 @@ next:
       { switch(tagex(key))
 	{ case TAG_VAR:                 c = T_TRY_VAR; break;
 	  case TAG_ATTVAR|STG_STATIC:   c = T_TRY_ATTVARA; break;
-	  case TAG_ATTVAR|STG_RESERVED: c = T_TRY_ATTVARZ; break;
+	  case TAG_ATTVAR|STG_RESERVED:
+	    if ( ison(state->trie, TRIE_ISTABLE) )
+	      c = T_TRY_ATTVARZT;
+	    else
+	      c = T_TRY_ATTVARZ;
+	    break;
 	  default: assert(0);           c = 0;
 	}
 	add_vmi_else_d(state, c, (code)index);
@@ -3180,7 +3192,12 @@ next:
       { switch(tagex(key))
 	{ case TAG_VAR:                 c = T_VAR; break;
 	  case TAG_ATTVAR|STG_STATIC:   c = T_ATTVARA; break;
-	  case TAG_ATTVAR|STG_RESERVED: c = T_ATTVARZ; break;
+	  case TAG_ATTVAR|STG_RESERVED:
+	    if ( ison(state->trie, TRIE_ISTABLE) )
+	      c = T_ATTVARZT;
+	    else
+	      c = T_ATTVARZ;
+	    break;
 	  default: assert(0);		c = 0;
 	}
 	add_vmi_d(state, c, (code)index);
