@@ -102,6 +102,7 @@ typedef struct trie_node
 #define TRIE_COMPLETE	0x0008		/* Answer trie is complete */
 #define TRIE_ABOLISH_ON_COMPLETE 0x0010	/* Abolish the table when completed */
 #define TRIE_ISTRACKED  0x0020		/* Trie changes are tracked */
+#define TRIE_ISTABLE	0x0040		/* Trie is an answer table */
 
 typedef struct trie
 { atom_t		symbol;		/* The associated symbol */
@@ -201,12 +202,12 @@ foreign_t trie_gen_raw(
 	      trie *trie, trie_node *root,
 	      term_t Key, term_t Value,
 	      term_t Data,
-	      int LDFUNCP (*unify_data)(term_t, trie_node*, void*),
+	      bool LDFUNCP (*unify_data)(term_t, trie_node*, void*),
 	      void *ctx, control_t PL__ctx);
 foreign_t clear_trie_gen_state(void *ctx);
 foreign_t trie_gen(term_t Trie, term_t Root, term_t Key, term_t Value,
 		   term_t Data,
-		   int LDFUNCP (*unify_data)(term_t, trie_node*, void*),
+		   bool LDFUNCP (*unify_data)(term_t, trie_node*, void*),
 		   void *ctx, control_t PL__ctx);
 void *	map_trie_node(trie_node *n,
 		      void* (*map)(trie_node *n, void *ctx), void *ctx);
